@@ -99,6 +99,11 @@ enum drbd_packet {
 	P_TWOPC_COMMIT        = 0x47, /* data sock: commit state change */
 	P_TWOPC_RETRY         = 0x48, /* meta sock: retry two-phase commit */
 
+	P_CONFIRM_STABLE      = 0x49, /* meta sock: similar to an unsolicited partial barrier ack */
+	P_RS_CANCEL_AHEAD     = 0x4a, /* protocol version 115,
+		 * meta: cancel RS_DATA_REQUEST packet if already Ahead again,
+		 *       tell peer to stop sending resync requests... */
+
 	P_MAY_IGNORE	      = 0x100, /* Flag to test if (cmd > P_MAY_IGNORE) ... */
 
 	/* special command ids for handshake */
@@ -291,6 +296,13 @@ struct p_barrier_ack {
 	uint32_t set_size;
 } __packed;
 
+struct p_confirm_stable {
+	uint64_t oldest_block_id;
+	uint64_t youngest_block_id;
+	uint32_t set_size;
+	uint32_t pad; /* to multiple of 8 Byte */
+} __packed;
+
 struct p_rs_param {
 	uint32_t resync_rate;
 
