@@ -11,7 +11,7 @@
    So that transport compiled against an older version of this
    header will no longer load in a module that assumes a newer
    version. */
-#define DRBD_TRANSPORT_API_VERSION 14
+#define DRBD_TRANSPORT_API_VERSION 16
 
 /* MSG_MSG_DONTROUTE and MSG_PROBE are not used by DRBD. I.e.
    we can reuse these flags for our purposes */
@@ -78,6 +78,7 @@ enum drbd_tr_free_op {
 	DESTROY_TRANSPORT
 };
 
+struct drbd_listener;
 
 /* A transport might wrap its own data structure around this. Having
    this base class as its first member. */
@@ -91,7 +92,12 @@ struct drbd_path {
 	int peer_addr_len;
 	bool established; /* updated by the transport */
 
-	struct list_head list;
+	struct list_head list; /* paths of a connection */
+	struct list_head listener_link; /* paths waiting for an incomming connection,
+					   head is in a drbd_listener */
+	struct drbd_listener *listener;
+
+	struct rcu_head rcu;
 };
 
 /* Each transport implementation should embed a struct drbd_transport
@@ -191,6 +197,7 @@ struct drbd_transport_class {
 	const char *name;
 	const int instance_size;
 	const int path_instance_size;
+	const int listener_instance_size;
 	struct module *module;
 	int (*init)(struct drbd_transport *);
 	struct list_head list;
@@ -204,23 +211,13 @@ struct drbd_listener {
 	struct kref kref;
 	struct drbd_resource *resource;
 	struct list_head list; /* link for resource->listeners */
-	struct list_head waiters; /* list head for waiter structs*/
+	struct list_head waiters; /* list head for paths */
 	spinlock_t waiters_lock;
 	int pending_accepts;
 	struct sockaddr_storage listen_addr;
 	void (*destroy)(struct drbd_listener *);
 };
 
-/* This represents a drbd receiver thread that is waiting for an
-   incoming connection attempt. Again, should be embedded into a
-   implementation object */
-struct drbd_waiter {
-	struct drbd_transport *transport;
-	wait_queue_head_t wait;
-	struct list_head list;
-	struct drbd_listener *listener;
-};
-
 /* drbd_main.c */
 extern void drbd_destroy_path(struct kref *kref);
 
@@ -233,11 +230,10 @@ extern struct drbd_transport_class *drbd_get_transport_class(const char *transpo
 extern void drbd_put_transport_class(struct drbd_transport_class *);
 extern void drbd_print_transports_loaded(struct seq_file *seq);
 
-extern int drbd_get_listener(struct drbd_waiter *waiter,
-			     const struct sockaddr *addr,
-			     int (*create_fn)(struct drbd_transport *, const struct sockaddr *, struct drbd_listener **));
-extern void drbd_put_listener(struct drbd_waiter *waiter);
-extern struct drbd_waiter *drbd_find_waiter_by_addr(struct drbd_listener *, struct sockaddr_storage *);
+extern int drbd_get_listener(struct drbd_transport *transport, struct drbd_path *path,
+			     int (*init_fn)(struct drbd_transport *, const struct sockaddr *, struct drbd_listener *));
+extern void drbd_put_listener(struct drbd_path *path);
+extern struct drbd_path *drbd_find_path_by_addr(struct drbd_listener *, struct sockaddr_storage *);
 extern bool drbd_stream_send_timed_out(struct drbd_transport *transport, enum drbd_stream stream);
 extern bool drbd_should_abort_listening(struct drbd_transport *transport);
 extern void drbd_path_event(struct drbd_transport *transport, struct drbd_path *path);
