@@ -80,7 +80,8 @@ enum drbd_after_sb_p {
 	ASB_CONSENSUS,
 	ASB_DISCARD_SECONDARY,
 	ASB_CALL_HELPER,
-	ASB_VIOLENTLY
+	ASB_VIOLENTLY,
+	ASB_RETRY_CONNECT,
 };
 
 enum drbd_on_no_data {
@@ -113,6 +114,11 @@ enum drbd_read_balancing {
 	RB_1M_STRIPING,
 };
 
+/* Windows km/dderror.h has that a 0L */
+#ifdef NO_ERROR
+#undef NO_ERROR
+#endif
+
 /* KEEP the order, do not delete or insert. Only append. */
 enum drbd_ret_code {
 	ERR_CODE_BASE		= 100,
@@ -362,7 +368,8 @@ enum mdf_peer_flag {
 	MDF_PEER_FENCING =	1 << 2,
 	MDF_PEER_FULL_SYNC =	1 << 3,
 	MDF_PEER_DEVICE_SEEN =	1 << 4,
-	MDF_NODE_EXISTS =       1 << 16, /* */
+	MDF_NODE_EXISTS =       1 << 16,
+	MDF_HAVE_BITMAP =       1 << 31,  /* For in core use; no meaning when persistet */
 };
 
 #define DRBD_PEERS_MAX 32
@@ -428,10 +435,6 @@ enum drbd_peer_state {
 #define DRBD_MD_MAGIC_84_UNCLEAN	(DRBD_MAGIC+5)
 #define DRBD_MD_MAGIC_09   (DRBD_MAGIC+6)
 
-/* how I came up with this magic?
- * base64 decode "actlog==" ;) */
-#define DRBD_AL_MAGIC 0x69cb65a2
-
 /* these are of type "int" */
 #define DRBD_MD_INDEX_INTERNAL -1
 #define DRBD_MD_INDEX_FLEX_EXT -2
