@@ -1,6 +1,9 @@
 #include <DrbdVolume.h>
 #include <DrbdConnection.h>
 #include <utils.h>
+#include <integerparse.h>
+
+#include "integerparse.h"
 
 const std::string DrbdVolume::PROP_KEY_VOL_NR      = "volume";
 const std::string DrbdVolume::PROP_KEY_MINOR       = "minor";
@@ -10,6 +13,7 @@ const std::string DrbdVolume::PROP_KEY_REPLICATION = "replication";
 const std::string DrbdVolume::PROP_KEY_CLIENT      = "client";
 const std::string DrbdVolume::PROP_KEY_PEER_CLIENT = "peer-client";
 const std::string DrbdVolume::PROP_KEY_QUORUM      = "quorum";
+const std::string DrbdVolume::PROP_KEY_SYNC_PERC   = "done";
 
 const char* DrbdVolume::DS_LABEL_DISKLESS     = "Diskless";
 const char* DrbdVolume::DS_LABEL_ATTACHING    = "Attaching";
@@ -46,6 +50,18 @@ const char* DrbdVolume::CS_LABEL_UNKNOWN              = "unknown";
 const char* DrbdVolume::QU_LABEL_PRESENT              = "yes";
 const char* DrbdVolume::QU_LABEL_LOST                 = "no";
 
+// Integer / Fraction separator
+const char* DrbdVolume::FRACT_SEPA = ".";
+
+// Maximum value of the sync_perc field - 100 * percent value -> 10,000
+const uint16_t DrbdVolume::MAX_SYNC_PERC = 10000;
+
+// Maximum percent value - 100
+const uint16_t DrbdVolume::MAX_PERC = 100;
+
+// Maximum fraction value - 2 digits precision, 99
+const uint16_t DrbdVolume::MAX_FRACT = 99;
+
 DrbdVolume::DrbdVolume(uint16_t volume_nr) :
     vol_nr(volume_nr)
 {
@@ -60,7 +76,12 @@ const uint16_t DrbdVolume::get_volume_nr() const
     return vol_nr;
 }
 
-// @throws EventMessageException
+uint16_t DrbdVolume::get_sync_perc() const
+{
+    return sync_perc;
+}
+
+// @throws std::bad_alloc, EventMessageException
 void DrbdVolume::update(PropsMap& event_props)
 {
     std::string* prop_disk = event_props.get(&PROP_KEY_DISK);
@@ -87,9 +108,11 @@ void DrbdVolume::update(PropsMap& event_props)
         {
             minor_nr = DrbdVolume::parse_minor_nr(*minor_nr_str);
         }
-        catch (NumberFormatException& nf_exc)
+        catch (dsaext::NumberFormatException&)
         {
-            throw EventMessageException();
+            std::string error_msg("Invalid DRBD event: Invalid minor number");
+            std::string debug_info("Invalid minor number");
+            throw EventMessageException(&error_msg, &debug_info, nullptr);
         }
     }
 
@@ -109,6 +132,69 @@ void DrbdVolume::update(PropsMap& event_props)
     {
         quorum_alert = !parse_quorum_state(*prop_quorum);
     }
+
+    {
+        bool is_resyncing {false};
+        switch (vol_repl_state)
+        {
+            case DrbdVolume::repl_state::OFF:
+                // fall-through
+            case DrbdVolume::repl_state::BEHIND:
+                // fall-through
+            case DrbdVolume::repl_state::STARTING_SYNC_TARGET:
+                // fall-through
+            case DrbdVolume::repl_state::SYNC_TARGET:
+                // fall-through
+            case DrbdVolume::repl_state::PAUSED_SYNC_TARGET:
+                // fall-through
+            case DrbdVolume::repl_state::VERIFY_TARGET:
+                is_resyncing = true;
+                break;
+            case DrbdVolume::repl_state::ESTABLISHED:
+                // Whenever the replication state returns to ESTABLISHED, reset
+                // the sync percentage, because the drbdsetup events commonly
+                // skips reporting 100 percent sync percentage
+                sync_perc = MAX_SYNC_PERC;
+                break;
+            case DrbdVolume::repl_state::AHEAD:
+                // fall-through
+            case DrbdVolume::repl_state::PAUSED_SYNC_SOURCE:
+                // fall-through
+            case DrbdVolume::repl_state::STARTING_SYNC_SOURCE:
+                // fall-through
+            case DrbdVolume::repl_state::SYNC_SOURCE:
+                // fall-through
+            case DrbdVolume::repl_state::VERIFY_SOURCE:
+                // fall-through
+            case DrbdVolume::repl_state::WF_BITMAP_SOURCE:
+                // fall-through
+            case DrbdVolume::repl_state::WF_BITMAP_TARGET:
+                // fall-through
+            case DrbdVolume::repl_state::WF_SYNC_UUID:
+                // fall-through
+            case DrbdVolume::repl_state::UNKNOWN:
+                // fall-through
+            default:
+                break;
+        }
+        if (is_resyncing)
+        {
+            std::string* prop_sync_perc = event_props.get(&PROP_KEY_SYNC_PERC);
+            if (prop_sync_perc != nullptr)
+            {
+                try
+                {
+                    sync_perc = parse_sync_perc(*prop_sync_perc);
+                }
+                catch (dsaext::NumberFormatException&)
+                {
+                    std::string error_msg("Invalid DRBD event: Invalid sync percentage value");
+                    std::string debug_info("Invalid sync percentage value");
+                    throw EventMessageException(&error_msg, &debug_info, nullptr);
+                }
+            }
+        }
+    }
 }
 
 int32_t DrbdVolume::get_minor_nr() const
@@ -116,7 +202,7 @@ int32_t DrbdVolume::get_minor_nr() const
     return minor_nr;
 }
 
-// @throws EventMessageException
+// @throws std::bad_alloc, EventMessageException
 void DrbdVolume::set_minor_nr(int32_t value)
 {
     if (value >= -1 && value < 0x100000)
@@ -125,7 +211,9 @@ void DrbdVolume::set_minor_nr(int32_t value)
     }
     else
     {
-        throw EventMessageException();
+        std::string error_msg("Invalid DRBD event: Invalid minor number");
+        std::string debug_info("Invalid minor number");
+        throw EventMessageException(&error_msg, &debug_info, nullptr);
     }
 }
 
@@ -407,7 +495,7 @@ StateFlags::state DrbdVolume::child_state_flags_changed()
     return StateFlags::state::NORM;
 }
 
-// @throws EventMessageException
+// @throws std::bad_alloc, EventMessageException
 DrbdVolume::disk_state DrbdVolume::parse_disk_state(std::string& state_name)
 {
     DrbdVolume::disk_state state = DrbdVolume::disk_state::UNKNOWN;
@@ -459,13 +547,15 @@ DrbdVolume::disk_state DrbdVolume::parse_disk_state(std::string& state_name)
     else
     if (state_name != DS_LABEL_UNKNOWN)
     {
-        throw EventMessageException();
+        std::string error_msg("Invalid DRBD event: Invalid disk state");
+        std::string debug_info("Invalid disk state");
+        throw EventMessageException(&error_msg, &debug_info, nullptr);
     }
 
     return state;
 }
 
-// @throws EventMessageException
+// @throws std::bad_alloc, EventMessageException
 DrbdVolume::repl_state DrbdVolume::parse_repl_state(std::string& state_name)
 {
     DrbdVolume::repl_state state = DrbdVolume::repl_state::UNKNOWN;
@@ -547,13 +637,15 @@ DrbdVolume::repl_state DrbdVolume::parse_repl_state(std::string& state_name)
     else
     if (state_name != RS_LABEL_UNKNOWN)
     {
-        throw EventMessageException();
+        std::string error_msg("Invalid DRBD event: Invalid replication state");
+        std::string debug_info("Invalid replication state");
+        throw EventMessageException(&error_msg, &debug_info, nullptr);
     }
 
     return state;
 }
 
-// @throws EventMessageException
+// @throws std::bad_alloc, EventMessageException
 DrbdVolume::client_state DrbdVolume::parse_client_state(std::string& value_str)
 {
     DrbdVolume::client_state state = DrbdVolume::client_state::UNKNOWN;
@@ -570,13 +662,15 @@ DrbdVolume::client_state DrbdVolume::parse_client_state(std::string& value_str)
     else
     if (value_str != CS_LABEL_UNKNOWN)
     {
-        throw EventMessageException();
+        std::string error_msg("Invalid DRBD event: Invalid client (diskless) mode");
+        std::string debug_info("Invalid client mode value");
+        throw EventMessageException(&error_msg, &debug_info, nullptr);
     }
 
     return state;
 }
 
-// @throws EventMessageException
+// @throws std::bad_alloc, EventMessageException
 bool DrbdVolume::parse_quorum_state(std::string& value_str)
 {
     bool quorum_present {false};
@@ -588,7 +682,9 @@ bool DrbdVolume::parse_quorum_state(std::string& value_str)
     else
     if (value_str != QU_LABEL_LOST)
     {
-        throw EventMessageException();
+        std::string error_msg("Invalid DRBD event: Invalid quorum state");
+        std::string debug_info("Invalid quorum state");
+        throw EventMessageException(&error_msg, &debug_info, nullptr);
     }
 
     return quorum_present;
@@ -598,17 +694,42 @@ bool DrbdVolume::parse_quorum_state(std::string& value_str)
 // @throws NumberFormatException
 uint16_t DrbdVolume::parse_volume_nr(std::string& value_str)
 {
-    uint16_t value {0};
-    value = NumberParser::parse_uint16(value_str);
-    return value;
+    return dsaext::parse_unsigned_int16(value_str);
 }
 
 // @throws NumberFormatException
 int32_t DrbdVolume::parse_minor_nr(std::string& value_str)
 {
-    int32_t value {-1};
-    value = NumberParser::parse_int32(value_str);
-    return value;
+    return dsaext::parse_signed_int32(value_str);
+}
+
+// @throws NumberFormatException
+uint16_t DrbdVolume::parse_sync_perc(std::string& value_str)
+{
+    uint16_t result {0};
+    size_t split_idx = value_str.find(FRACT_SEPA, 0);
+    if (split_idx != std::string::npos)
+    {
+        std::string perc_value_str = value_str.substr(0, split_idx);
+        std::string perc_fract_str = value_str.substr(split_idx + 1);
+
+        uint16_t value = dsaext::parse_unsigned_int16(perc_value_str);
+        if (value > MAX_PERC)
+        {
+            throw dsaext::NumberFormatException();
+        }
+        uint16_t fract = dsaext::parse_unsigned_int16(perc_fract_str);
+        if (fract > MAX_FRACT)
+        {
+            throw dsaext::NumberFormatException();
+        }
+        result = value * 100 + fract;
+        if (result > MAX_SYNC_PERC)
+        {
+            throw dsaext::NumberFormatException();
+        }
+    }
+    return result;
 }
 
 // Creates (allocates and initializes) a new DrbdVolume object from a map of properties
@@ -624,10 +745,10 @@ DrbdVolume* DrbdVolume::new_from_props(PropsMap& event_props)
     {
         try
         {
-            uint16_t vol_nr = NumberParser::parse_uint16(*number_str);
+            uint16_t vol_nr = dsaext::parse_unsigned_int16(*number_str);
             vol = new DrbdVolume(vol_nr);
         }
-        catch (NumberFormatException& nf_exc)
+        catch (dsaext::NumberFormatException&)
         {
             // no-op
         }
