@@ -7,13 +7,13 @@
 #include <linux/genetlink.h>
 
 #include "libgenl.h"
-#include <linux/drbd.h>
-#include <linux/drbd_config.h>
-#include <linux/drbd_genl_api.h>
-#include <linux/drbd_limits.h>
+#include "linux/drbd.h"
+#include "linux/drbd_config.h"
+#include "linux/drbd_genl_api.h"
+#include "linux/drbd_limits.h"
 #include "drbd_nla.h"
 #include "drbdtool_common.h"
-#include <linux/genl_magic_func.h>
+#include "linux/genl_magic_func.h"
 #include "config_flags.h"
 
 #ifndef ARRAY_SIZE
@@ -799,6 +799,7 @@ const char *rr_conflict_map[] = {
 	[ASB_DISCONNECT] = "disconnect",
 	[ASB_VIOLENTLY] = "violently",
 	[ASB_CALL_HELPER] = "call-pri-lost",
+	[ASB_RETRY_CONNECT] = "retry-connect",
 };
 
 const char *on_no_data_map[] = {
@@ -896,6 +897,7 @@ const struct en_map quorum_map[] = {
 	{ "socket-check-timeout", NUMERIC(sock_check_timeo, SOCKET_CHECK_TIMEO) },	\
 	{ "fencing", ENUM(fencing_policy, FENCING) },					\
 	{ "max-buffers", NUMERIC(max_buffers, MAX_BUFFERS) },				\
+	{ "allow-remote-read", BOOLEAN(allow_remote_read, ALLOW_REMOTE_READ) },					\
 	{ "_name", STRING(name) }
 
 struct context_def disk_options_ctx = {
@@ -1029,6 +1031,7 @@ struct context_def new_current_uuid_cmd_ctx = {
 	.nla_type = DRBD_NLA_NEW_C_UUID_PARMS,
 	.fields = {
 		{ "clear-bitmap", FLAG(clear_bm) },
+		{ "force-resync", FLAG(force_resync) },
 		{ } },
 };
 
