@@ -37,6 +37,7 @@
 #include <assert.h>
 
 #include <sys/types.h>
+#include <linux/types.h>
 #include <sys/wait.h>
 #include <poll.h>
 #include <sys/socket.h>
@@ -610,6 +611,8 @@ static int __call_cmd_fn(const struct cfg_ctx *ctx, enum on_error on_error)
 		struct path *path;
 
 		for_each_path(path, &tmp_ctx.conn->paths) {
+			if (path->ignore)
+				continue;
 			tmp_ctx.path = path;
 			rv = tmp_ctx.cmd->function(&tmp_ctx);
 			if (rv >= 20) {
@@ -702,6 +705,7 @@ static char *drbd_cfg_stage_string[] = {
 	[CFG_NET_DISCONNECT] = "prepare net",
 	[CFG_NET_PREP_DOWN] = "prepare net",
 	[CFG_NET_PREP_UP] = "prepare net",
+	[CFG_NET_PATH] = "prepare net",
 	[CFG_NET] = "adjust net",
 	[CFG_PEER_DEVICE] = "adjust peer_devices",
 	[CFG_NET_CONNECT] = "attempt to connect",
@@ -1127,7 +1131,12 @@ static int adm_attach(const struct cfg_ctx *ctx)
 	bool reset = (ctx->cmd == &disk_options_defaults_cmd);
 
 	if (do_attach) {
-		int rv = call_cmd_fn(&apply_al_cmd, ctx, KEEP_RUNNING);
+		int rv;
+		rv = before_attach(ctx);
+		if (rv)
+			return rv;
+
+		rv = call_cmd_fn(&apply_al_cmd, ctx, KEEP_RUNNING);
 		if (rv)
 			return rv;
 	}
@@ -1146,18 +1155,14 @@ static int adm_attach(const struct cfg_ctx *ctx)
 		}
 		argv[NA(argc)] = vol->meta_index;
 	}
-	if (reset)
+	if (reset) {
+		struct d_option *option;
 		argv[NA(argc)] = "--set-defaults";
-	if (reset || do_attach) {
-		if (!do_attach) {
-			struct d_option *option;
-			STAILQ_FOREACH(option, &ctx->vol->disk_options, link)
-				if (!option->adj_skip)
-					make_option(argv[NA(argc)], option);
-		} else {
-			make_options(argv[NA(argc)], &ctx->vol->disk_options);
-		}
-	}
+		STAILQ_FOREACH(option, &ctx->vol->disk_options, link)
+			if (!option->adj_skip)
+				make_option(argv[NA(argc)], option);
+	} else
+		make_options(argv[NA(argc)], &ctx->vol->disk_options);
 	add_setup_options(argv, &argc, ctx->cmd->drbdsetup_ctx);
 	argv[NA(argc)] = 0;
 
@@ -1206,6 +1211,9 @@ int adm_new_minor(const struct cfg_ctx *ctx)
 	if (!ex && do_register)
 		register_minor(ctx->vol->device_minor, config_save);
 
+	if (!ex)
+		ex = after_new_minor(ctx);
+
 	return ex;
 }
 
@@ -1419,6 +1427,12 @@ static void __adm_drbdsetup(const struct cfg_ctx *ctx, int flags, pid_t *pid, in
 		setenv("DRBD_RESOURCE", ctx->res->name, 1);
 
 	m__system(argv, flags, ctx->res ? ctx->res->name : NULL, pid, fd, ex);
+
+	if (ctx->cmd == &primary_cmd)
+		after_primary(ctx);
+
+	if (ctx->cmd == &secondary_cmd || ctx->cmd == &down_cmd)
+		after_secondary(ctx);
 }
 
 static int _adm_drbdsetup(const struct cfg_ctx *ctx, int flags)
@@ -1610,7 +1624,7 @@ static int adm_khelper(const struct cfg_ctx *ctx)
 	char *sh_cmd;
 	char minor_string[8];
 	char volume_string[8];
-	char *argv[] = { "/bin/sh", "-c", NULL, NULL };
+	char *argv[4] = { NULL, };
 
 	setenv("DRBD_CONF", config_save, 1);
 	setenv("DRBD_RESOURCE", res->name, 1);
@@ -1678,7 +1692,11 @@ static int adm_khelper(const struct cfg_ctx *ctx)
 	}
 
 	if ((sh_cmd = get_opt_val(&res->handlers, ctx->cmd->name, NULL))) {
+		argv[0] = khelper_argv[0];
+		argv[1] = khelper_argv[1];
 		argv[2] = sh_cmd;
+		argv[3] = NULL;
+
 		rv = m_system_ex(argv, SLEEPS_VERY_LONG, res->name);
 	}
 	return rv;
@@ -2139,7 +2157,7 @@ int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check)
 
 	/* resource name only (e.g., r0) and in check state
 	 * this would enable all connectionst that are not ignored */
-	if (!conn_or_hostname && check == WOULD_ENABLE_MULTI_TIMES)
+	if (!vol_id && !conn_or_hostname && check == WOULD_ENABLE_MULTI_TIMES)
 		for_each_connection(conn, &res->connections)
 			if (!conn->ignore) {
 				if (!conn->ignore_tmp)
@@ -2799,18 +2817,10 @@ char *canonify_path(char *path)
 
 void assign_command_names_from_argv0(char **argv)
 {
-	struct cmd_helper {
-		char *name;
-		char **var;
-	};
-	static struct cmd_helper helpers[] = {
-		{"drbdsetup", &drbdsetup},
-		{"drbdmeta", &drbdmeta},
-		{"drbd-proxy-ctl", &drbd_proxy_ctl},
-		{"drbdadm-83", &drbdadm_83},
-		{"drbdadm-84", &drbdadm_84},
-		{NULL, NULL}
-	};
+	/* The predefined list of command helpers has been moved to
+	 * platform-dependent code.
+	 */
+
 	struct cmd_helper *c;
 
 	/* in case drbdadm is called with an absolute or relative pathname
@@ -3009,6 +3019,7 @@ int parse_options(int argc, char **argv, struct adm_cmd **cmd, char ***resource_
 			printf("DRBD_KERNEL_VERSION=%s\n", escaped_version_code_kernel());
 			printf("DRBDADM_VERSION_CODE=0x%06x\n", version_code_userland());
 			printf("DRBDADM_VERSION=%s\n", shell_escape(PACKAGE_VERSION));
+			print_platform_specific_versions();
 			exit(0);
 			break;
 		case 'P':
@@ -3045,7 +3056,7 @@ int parse_options(int argc, char **argv, struct adm_cmd **cmd, char ***resource_
 	}
 
 	if (help)
-		print_usage_and_exit(*cmd, NULL, 0);
+		print_usage_and_exit(*cmd, NULL, (*cmd == NULL) ? E_USAGE: 0);
 
 	if (*cmd == NULL) {
 		if (first_arg_index < argc) {
@@ -3248,6 +3259,7 @@ int main(int argc, char **argv)
 	}
 
 	assign_command_names_from_argv0(argv);
+	maybe_add_bin_dir_to_path();
 
 	if (drbdsetup == NULL || drbdmeta == NULL || drbd_proxy_ctl == NULL) {
 		err("could not strdup argv[0].\n");
@@ -3267,12 +3279,15 @@ int main(int argc, char **argv)
 		exit(E_USAGE);
 	}
 
-	do_verify_ips = cmd->verify_ips;
-
 	is_dump_xml = (cmd == &dump_xml_cmd);
 	is_dump = (is_dump_xml || cmd == &dump_cmd);
 	is_adjust = (cmd == &adjust_cmd || cmd == &adjust_wp_cmd);
 
+	if (is_adjust && find_backend_option("--skip-net"))
+		do_verify_ips = 0;
+	else
+		do_verify_ips = cmd->verify_ips;
+
 	if (!resource_names[0]) {
 		if (!is_dump && cmd->res_name_required)
 			print_usage_and_exit(cmd, "No resource names specified", E_USAGE);
@@ -3418,8 +3433,6 @@ int main(int argc, char **argv)
 				adjust_more_than_one_resource = 1;
 
 			for (i = 0; resource_names[i]; i++) {
-				ctx.res = NULL;
-				ctx.vol = NULL;
 				r = ctx_by_name(&ctx, resource_names[i], SETUP_MULTI);
 				if (!ctx.res) {
 					ctx_by_minor(&ctx, resource_names[i]);
