@@ -60,11 +60,7 @@ void set_on_hosts_in_res(struct d_resource *res)
 
 	for_each_host(host, &res->all_hosts) {
 		if (host->lower_name) {
-			for_each_resource(l_res, &config) {
-				if (!strcmp(l_res->name, host->lower_name))
-					break;
-			}
-
+			l_res = res_by_name(host->lower_name);
 			if (l_res == NULL) {
 				err("%s:%d: in resource %s, "
 				    "referenced resource '%s' not defined.\n",
@@ -97,7 +93,7 @@ void set_on_hosts_in_res(struct d_resource *res)
 			host->lower = l_res;
 
 			/* */
-			if (addr_scope_local(host->address.addr))
+			if (host->address.addr && addr_scope_local(host->address.addr))
 				STAILQ_FOREACH(h, &host->on_hosts, link)
 					check_uniq("IP", "%s:%s:%s", h->name, host->address.addr, host->address.port);
 
@@ -369,6 +365,13 @@ void set_me_in_resource(struct d_resource* res, int match_on_proxy)
 			}
 
 			if (h) {
+				/* In case this is a path of a stacked resource ignore it if
+				   the path belongs to the "non local" node of the lower resource.
+				   Never ignore an implicit path, since they have faked hostnames */
+				if (!h->faked_hostname && h->host_info->lower &&
+				    !hostname_in_list(h->name, &h->host_info->lower->me->on_hosts))
+					path->ignore = 1;
+
 				h->used_as_me = 1;
 				if (!path->my_address)
 					path->my_address = h->address.addr ? &h->address : &res->me->address;
@@ -639,9 +642,14 @@ static void check_volume_complete(struct d_resource *res, struct d_host_info *ho
 		if (!(vol->disk && !strcmp(vol->disk, "none"))) {
 			if (!vol->disk)
 				derror(host, res, "disk");
-			if (!vol->meta_disk)
+			if (!vol->meta_disk && !vol->meta_index) {
+				/* implicit internal */
+				vol->meta_disk = strdup("internal");
+				vol->meta_index = strdup("internal");
+			}
+			else if (!vol->meta_disk)
 				derror(host, res, "meta-disk");
-			if (!vol->meta_index)
+			else if (!vol->meta_index)
 				derror(host, res, "meta-index");
 		}
 	}
@@ -779,7 +787,7 @@ static void create_implicit_connections(struct d_resource *res)
 	for_each_host(host_info, &res->all_hosts) {
 		if (++hosts == 3) {
 			err("Resource %s:\n\t"
-			    "Use explicit 'connection' sections with more than two 'on' sections.\n",
+			    "Use explicit 'connection' sections (or a 'connection-mesh' section) with more than two 'on' sections.\n",
 		            res->name);
 			break;
 		}
@@ -1087,6 +1095,11 @@ static void fixup_peer_devices(struct d_resource *res)
 		}
 
 		some_host = STAILQ_FIRST(&some_path->hname_address_pairs)->host_info;
+		if (!some_host) {
+			/* oops? not even one single host?? */
+			assert(config_valid == 0);
+			continue;
+		}
 		for_each_volume(vol, &some_host->volumes) {
 			peer_device = find_peer_device(conn, vol->vnr);
 			if (peer_device)
@@ -1129,7 +1142,7 @@ void post_parse(struct resources *resources, enum pp_flags flags)
 		check_volumes_hosts(res);
 
 		if (any_implicit && any_non_zero_vnr) {
-			err("%s:%d: in resource %s: you must not mix implicit any explicit volumes\n",
+			err("%s:%d: in resource %s: you must not mix implicit and explicit volumes\n",
 			    config_file, line, res->name);
 			config_valid = 0;
 		}
@@ -1318,6 +1331,10 @@ void expand_common(void)
 
 			STAILQ_FOREACH(ha, &some_path->hname_address_pairs, link) {
 				h = ha->host_info;
+				if (!h) {
+					assert(config_valid == 0);
+					continue;
+				}
 				for_each_volume(vol, &h->volumes) {
 					peer_device = find_peer_device(conn, vol->vnr);
 
@@ -1339,6 +1356,16 @@ struct d_resource *res_by_name(const char *name)
 	return NULL;
 }
 
+static struct d_resource *res_by_name_ign_vol(const char *name)
+{
+	char *name_dup = strdupa(name);
+	char *slash = strchr(name_dup, '/');
+
+	if (slash) *slash = '\0';
+
+	return res_by_name(name_dup);
+}
+
 static int sanity_check_abs_cmd(char *cmd_name)
 {
 	struct stat sb;
@@ -1508,7 +1535,7 @@ static void validate_resource(struct d_resource *res, enum pp_flags flags)
 	next:
 		if (strcmp(opt->name, "resync-after"))
 			continue;
-		rs_after_res = res_by_name(opt->value);
+		rs_after_res = res_by_name_ign_vol(opt->value);
 		if (rs_after_res == NULL ||
 		    (rs_after_res->ignore && !(flags & MATCH_ON_PROXY))) {
 			err("%s:%d: in resource %s:\n\tresource '%s' mentioned in "
@@ -1606,7 +1633,7 @@ static void _convert_after_option(struct d_resource *res, struct d_volume *vol)
 	next:
 		if (strcmp(opt->name, "resync-after"))
 			continue;
-		ctx_by_name(&depends_on_ctx, opt->value, CTX_FIRST);
+		ctx_by_name(&depends_on_ctx, opt->value, SETUP_MULTI);
 		volumes = ctx_set_implicit_volume(&depends_on_ctx);
 		if (volumes > 1) {
 			err("%s:%d: in resource %s:\n\t"
@@ -1617,15 +1644,28 @@ static void _convert_after_option(struct d_resource *res, struct d_volume *vol)
 			return;
 		}
 
-		if (!depends_on_ctx.res || depends_on_ctx.res->ignore) {
-			struct d_option *next = STAILQ_NEXT(opt, link);
-			STAILQ_REMOVE(&vol->disk_options, opt, d_option, link);
-			free_opt(opt);
-			opt = next;
-			if (opt)
-				goto next;
-			else
-				break;
+		if (!depends_on_ctx.res || depends_on_ctx.res->ignore || !depends_on_ctx.vol) {
+			err("%s:%d: in resource %s:\n\tresource '%s' mentioned in "
+			    "'resync-after' option is not known%s.\n",
+			    res->config_file, res->start_line, res->name,
+			    opt->value, depends_on_ctx.res ? " on this host" : "");
+			/* Non-fatal if run from some script.
+			 * When deleting resources, it is an easily made
+			 * oversight to leave references to the deleted
+			 * resources in resync-after statements.  Don't fail on
+			 * every pacemaker-induced action, as it would
+			 * ultimately lead to all nodes committing suicide. */
+			if (no_tty) {
+				struct d_option *next = STAILQ_NEXT(opt, link);
+				STAILQ_REMOVE(&vol->disk_options, opt, d_option, link);
+				free_opt(opt);
+				opt = next;
+				if (opt)
+					goto next;
+				else
+					break;
+			} else
+				config_valid = 0;
 		} else {
 			free(opt->value);
 			m_asprintf(&opt->value, "%d", depends_on_ctx.vol->device_minor);
