@@ -39,6 +39,7 @@
 #include <netinet/in.h>
 #include <netdb.h>
 
+#include "config.h"
 #include "drbdadm.h"
 #include "drbdtool_common.h"
 #include "drbd_endian.h"
@@ -47,7 +48,7 @@
 
 #define HTTP_PORT 80
 #define HTTP_HOST "usage.drbd.org"
-#define HTTP_ADDR "212.69.161.111"
+#define HTTP_ADDR "159.69.154.96"
 #define NODE_ID_FILE DRBD_LIB_DIR"/node_id"
 
 
@@ -222,9 +223,11 @@ struct hostent *my_gethostbyname(const char *name)
 	sigaction(SIGALRM, &sa, &so);
 
 	if (!sigsetjmp(timed_out, 1)) {
+#ifdef HAVE_GETHOSTBYNAME_R
 		struct hostent ret;
 		char buf[2048];
 		int my_h_errno;
+#endif
 
 		alarm(DNS_TIMEOUT);
 		/* h = gethostbyname(name);
@@ -237,7 +240,11 @@ struct hostent *my_gethostbyname(const char *name)
 		 * gethostbyname_r() apparently does not use any internal locks.
 		 * Even if unnecessary in our case, it feels less dirty.
 		 */
+#ifdef HAVE_GETHOSTBYNAME_R
 		gethostbyname_r(name, &ret, buf, sizeof(buf), &h, &my_h_errno);
+#else
+		h = gethostbyname(name);
+#endif
 	} else {
 		/* timed out, longjmp of SIGALRM jumped here */
 		h = NULL;
@@ -389,7 +396,6 @@ void uc_node(enum usage_count_type type)
 {
 	struct node_info ni;
 	char *uri;
-	int send = 0;
 	int update = 0;
 	char answer[ANSWER_SIZE];
 	char n_comment[ANSWER_SIZE*3];
@@ -400,12 +406,12 @@ void uc_node(enum usage_count_type type)
 	if( getuid() != 0 ) return;
 
 	/* not when running directly from init,
-	 * or if stdout is no tty.
+	 * or if stdout is no tty and type is ask.
 	 * you do not want to have the "user information message"
 	 * as output from `drbdadm sh-resources all`
 	 */
 	if (getenv("INIT_VERSION")) return;
-	if (no_tty) return;
+	if (no_tty && type == UC_ASK) return;
 
 	/* If we don't know the current version control system hash,
 	 * we found the version via "modprobe -F version drbd",
@@ -421,14 +427,10 @@ void uc_node(enum usage_count_type type)
 	if( ! read_node_id(&ni) ) {
 		get_random_bytes(&ni.node_uuid,sizeof(ni.node_uuid));
 		ni.rev = *driver_version;
-		send = 1;
 	} else if (driver_version && !version_equal(&ni.rev, driver_version)) {
 		ni.rev = *driver_version;
 		update = 1;
-		send = 1;
-	}
-
-	if(!send) return;
+	} else return;
 
 	n_comment[0]=0;
 	if (type == UC_ASK ) {
@@ -446,7 +448,7 @@ void uc_node(enum usage_count_type type)
 		    "http://"HTTP_HOST"/cgi-bin/insert_usage.pl?nu="U64"&%s\n\n"
 		    "In case you want to participate but know that this machine is firewalled,\n"
 		    "simply issue the query string with your favorite web browser or wget.\n"
-		    "You can control all of this by setting 'usage-count' in your drbd.conf.\n\n"
+		    "You can control all of this by setting 'usage-count' in your global_common.conf.\n\n"
 		    "* You may enter a free form comment about your machine, that gets\n"
 		    "  used on "HTTP_HOST" instead of the big random number.\n"
 		    "* If you wish to opt out entirely, simply enter 'no'.\n"
@@ -454,7 +456,7 @@ void uc_node(enum usage_count_type type)
 		    update ? "an update" : "a new installation",
 		    PACKAGE_VERSION, ni.node_uuid, vcs_to_str(&ni.rev));
 		r = fgets(answer, ANSWER_SIZE, stdin);
-		if(r && !strcmp(answer,"no\n")) send = 0;
+		if(r && !strcmp(answer,"no\n")) return;
 		url_encode(answer,n_comment);
 	}
 
@@ -462,23 +464,20 @@ void uc_node(enum usage_count_type type)
 		       ni.node_uuid, vcs_to_str(&ni.rev),
 		       n_comment[0] ? "&nc=" : "", n_comment);
 
-	if (send) {
-		write_node_id(&ni);
+	write_node_id(&ni);
 
-		err("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"
-		    "  --==  Thank you for participating in the global usage survey  ==--\n"
-		    "The server's response is:\n\n");
-		make_get_request(uri);
-		if (type == UC_ASK) {
-			err("\n"
-			    "From now on, drbdadm will contact "HTTP_HOST" only when you update\n"
-			    "DRBD or when you use 'drbdadm create-md'. Of course it will continue\n"
-			    "to ask you for confirmation as long as 'usage-count' is at its default\n"
-			    "value of 'ask'.\n\n"
-			    "Just press [RETURN] to continue: ");
-			if (fgets(answer, 9, stdin) == NULL)
-				err("Could not read answer\n");
-		}
+	err("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n"
+			"  --==  Thank you for participating in the global usage survey  ==--\n"
+			"The server's response is:\n\n");
+	make_get_request(uri);
+	if (type == UC_ASK) {
+		err("\n"
+				"From now on, drbdadm will contact "HTTP_HOST" only when you update\n"
+				"DRBD or when you use 'drbdadm create-md'. Of course it will continue\n"
+				"to ask you for confirmation as long as 'usage-count' is set to 'ask'\n\n"
+				"Just press [RETURN] to continue: ");
+		if (fgets(answer, 9, stdin) == NULL)
+			err("Could not read answer\n");
 	}
 }
 
